<?php

namespace Drupal\unomi;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Routing\UrlGeneratorInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\unomi\UnomiConnector\UnomiConnectorPluginManager;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Unomi Integration service.
 */
class UnomiApi implements UnomiApiInterface {

  use StringTranslationTrait;

  /**
   * The Dropsolid Personalization config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $unomiConfig;

  /**
   * Drupal state.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The URL generator.
   *
   * @var \Drupal\Core\Routing\UrlGeneratorInterface
   */
  protected $urlGenerator;

  /**
   * The current request object.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The messenger to send info or warnings to Drupal with.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Unomi Connector Plugin Manager.
   *
   * @var \Drupal\unomi\UnomiConnector\UnomiConnectorPluginManager
   */
  protected $unomiConnectorPluginManager;

  /**
   * Constructs a UnomiApi class.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state.
   * @param \Drupal\Core\Routing\UrlGeneratorInterface $url_generator
   *   The URL Generator service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger to send info or warnings to Drupal with.
   * @param \Drupal\unomi\UnomiConnector\UnomiConnectorPluginManager $unomi_connector_plugin_manager
   *   The Unomi Connector Plugin Manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, StateInterface $state, UrlGeneratorInterface $url_generator, RequestStack $request_stack, MessengerInterface $messenger, UnomiConnectorPluginManager $unomi_connector_plugin_manager) {
    $this->unomiConfig = $config_factory->get('unomi.settings');
    $this->state = $state;
    $this->urlGenerator = $url_generator;
    $this->request = $request_stack->getCurrentRequest();
    $this->messenger = $messenger;
    $this->unomiConnectorPluginManager = $unomi_connector_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function getClient() {
    if ($this->unomiConfig->get('connector') && $this->unomiConfig->get('connector_config')) {
      // Ask the plugin to give us the Unomiclient.
      /** @var \Drupal\unomi\UnomiConnectorInterface $connector */
      $connector = $this->unomiConnectorPluginManager->createInstance($this->unomiConfig->get('connector'), $this->unomiConfig->get('connector_config'));
      return $connector->getClient();
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getApiClient() {
    if ($this->unomiConfig->get('connector') && $this->unomiConfig->get('connector_config')) {

      // Ask the plugin to give us the Unomiclient.
      /** @var \Drupal\unomi\UnomiConnectorInterface $connector */
      $connector = $this->unomiConnectorPluginManager->createInstance($this->unomiConfig->get('connector'), $this->unomiConfig->get('connector_config'));
      return $connector->getApiClient();
    }
    return NULL;
  }

}
